function gvm = rgvmises1a(n,m1,m2,k1,k2); 

%_____________________________________________________________________________
% RGVMISES1a.M
% arguments: n (number of generations),  m1, m2, k1, k2 
% This function generates GvM(m1,m2,k1,k2) pseudo-random numbers with the 
% ratio-of-uniforms method
% The sampling domain is optimized (defined by the numerical search of a)
%_____________________________________________________________________________

% Upper bounds 
aup=exp((k1+k2)/2);
bup=2*pi*aup;

% Determination of a
d=mod(m1-m2,pi);
sd=sin(d); cd=cos(d);
r=k1/4/k2;
coef=[1,-4*r*sd*cd,r^2-1,2*r*sd*cd,(sd*cd)^2];
rts=roots(coef);
t=[asin(rts);pi-asin(rts)];
st=sin(t); ct=cos(t);
f=(1-2*sd^2)*st.*ct-2*sd*cd*st.^2+r*st+sd*cd;
af=abs(f);
tsel=[t(af<.01);0];
cos1=cos(tsel); cos2=cos(2*(tsel+d));
g2=(k1*cos1+k2*cos2)/2;
a=max(exp(g2));  

% Determination of b
b=2*pi*a;

% Generation in the triangle
rand('state',0) % the initial state is fixed
U=rand(n,2);
V=b*U(:,2);
U(:,2)=[];
U=a*U;
sel=(V>2*pi*U);
U(sel)=a-U(sel);
V(sel)=b-V(sel);

% Final computations
R=V./U;
W=(k1*cos(R-m1)+k2*cos(2*(R-m2)))/2;
gvm=R(U<=exp(W));

