function gvm=rgvmises1s(n,m1,m2,k1,k2); 

%_____________________________________________________________________________
% RGVMISES1S.M
% arguments: n (number of generations),  m1, m2, k1, k2 
% This function generates GvM(m1,m2,k1,k2) pseudo-random numbers with the 
% ratio-of-uniforms method
% The sampling domain is non-optimal (it defined by the upper bounds for a and 
% b, the sampling is done over a rectangle instead of a triangle) 
% Squeezing is applied, k1+k2>=2 is required
%_____________________________________________________________________________

if (k1+k2>=2),
  disp('Error: squeezing impossible');
  gvm=[];
  return; 
end;

% Determination of the sampling domain
a=exp((k1+k2)/2);
b=2*pi*a;

% Generation in the sampling domain
rand('state',0) % the initial state is fixed
U=rand(n,2)*[a 0;0 b];

% Selection  
sel1=(U(:,2)<2*pi*U(:,1))&(U(:,1)<=1/(1-(k1+k2)/2));
Usel1=U(sel1,1);
R=U(sel1,2)./Usel1;
clear U;

% First acceptance 
sel2=(Usel1<=1-(k1+k2)/2); 
sum(sel2)
gvm=R(sel2);

% Second acceptance
Rnsel2=R(~sel2);
clear R;
W=(k1*cos(Rnsel2-m1)+k2*cos(2*(Rnsel2-m2)))/2;
sel3=(Usel1(~sel2)<=exp(W));

gvm=[gvm;Rnsel2(sel3)];

