function gvm = rgvmises2(n,m1,m2,k1,k2); 

%_____________________________________________________________________________
% RGVMISES2.M
% arguments: n (number of generations),  m1, m2, k1, k2 
% This function generates GvM(m1,m2,k1,k2) pseudo-random numbers with the 
% von Neumann method
% The sampling domain is determined by the numerical search of m
%_____________________________________________________________________________

% Determination of m
d=mod(m1-m2,pi);
sd=sin(d); cd=cos(d);
r=k1/4/k2;
coef=[1,-4*r*sd*cd,r^2-1,2*r*sd*cd,(sd*cd)^2];
rts=roots(coef);
t=[asin(rts);pi-asin(rts)];
st=sin(t); ct=cos(t);
f=(1-2*sd^2)*st.*ct-2*sd*cd*st.^2+r*st+sd*cd;
af=abs(f);
tsel=[t(af<.01);0];
cos1=cos(tsel); cos2=cos(2*(tsel+d));
m=max(exp(k1*cos1+k2*cos2));

% Generation in the sampling domain
rand('state',0) % the initial state is fixed
U=rand(n,2);
U1=U(:,1)*2*pi;
U2=U(:,2)*m;

% Final computations
sel=(U2<exp(k1*cos(U1-m1)+k2*cos(2*(U1-m2))));
gvm=U1(sel);

